﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Sociometry
{
    public partial class CountKeysForm : Form
    {
        public CountKeysForm()
        {
            InitializeComponent();
        }

        private void buttonOpenFile_Click(object sender, EventArgs e)
        {
            OpenFileDialog fileDialog = new OpenFileDialog
            {
                InitialDirectory = AppDomain.CurrentDomain.BaseDirectory,
                Filter = "Текстовые файлы|*.txt"
            };
            if (fileDialog.ShowDialog() == DialogResult.OK)
            {
                textBoxFileName.Text = fileDialog.FileName;
            }
        }

        private void buttonCountKeys_Click(object sender, EventArgs e)
        {
            if (textBoxFileName.Text == "")
            {
                MessageBox.Show("Не выбран файл для анализа!");
            }
            else if (dataGridKeys.Rows.Count == 0)
            {
                MessageBox.Show("Не заданы слова-ключи для анализа!");
            }
            else
            {
                CountKeyFrequency();
            }
        }

        private void CountKeyFrequency()
        {
            string fileName = textBoxFileName.Text;
            int symbolsCount = CountSymbols(fileName);
            string[] words = WordsFromText(fileName);
            string key;
            double keyQuantity;
            double frequency;
            ClearDataGrid();
            dataGridKeys.CurrentCell = dataGridKeys.Rows[0].Cells[0];
            for (int i = 0; i < dataGridKeys.Rows.Count; i++)
            {
                key = Convert.ToString(dataGridKeys.Rows[i].Cells["keys"].Value);
                keyQuantity = CountWordQuantity(key, words);
                frequency = Math.Round(keyQuantity * 10000 / symbolsCount, 2);
                dataGridKeys.Rows[i].Cells["frequency"].Value = Convert.ToString(frequency);
            }
        }

        private void ClearDataGrid()
        {
            for (int i = 0; i < dataGridKeys.Rows.Count; i++)
            {
                dataGridKeys.Rows[i].Cells["frequency"].Value = "";
                string keyValue = Convert.ToString(dataGridKeys.Rows[i].Cells["keys"].Value);
                if (keyValue==null||keyValue.Trim()=="")
                {
                    dataGridKeys.Rows.RemoveAt(i);
                    i--;
                }
            }
        }

        private int CountSymbols(string fileName)
        {
            string fileText = File.ReadAllText(fileName, Encoding.GetEncoding(1251));
            fileText = Regex.Replace(fileText, @"\p{C}+", string.Empty);
            return fileText.Length;
        }

        private string[] WordsFromText(string fileName)
        {
            string fileText = File.ReadAllText(fileName, Encoding.GetEncoding(1251));
            fileText = Regex.Replace(fileText.ToLower(), @"\p{C}+", " ");
            //fileText = Regex.Replace(fileText.ToLower(), @"\W+", "", RegexOptions.IgnoreCase);
            string[] words = Regex.Matches(fileText, @"\w+")
                    .Cast<Match>()
                    .Select(x => x.Value)
                    .ToArray();
            //words = fileText.Split(new char[] { ' ', '.', ',', '!', '?', ';' });
            return words;
        }

        private int CountWordQuantity(string word, string[] words)
        {
            int count = 0;
            for (int i = 0; i < words.Length; i++)
            {
                if (words[i] == word.ToLower())
                {
                    count++;
                }
            }
            return count;
        }

        private void buttonAddKey_Click(object sender, EventArgs e)
        {
            dataGridKeys.Rows.Add();
        }

        private void buttonDeleteRow_Click(object sender, EventArgs e)
        {
            if (dataGridKeys.SelectedCells.Count > 0)
            {
                DialogResult result = MessageBox.Show("Удалить строку?", "Удаление записи", MessageBoxButtons.YesNo);
                if (result == DialogResult.Yes)
                {
                    dataGridKeys.Rows.Remove(dataGridKeys.CurrentRow);
                }
            }
        }
    }
}
