﻿
using System;
using System.Security.Cryptography;

namespace Sociometry
{
    class Key
    {
        public string Name { get; set; }
        public double[] Sigma { get; set; }
        public double[] A { get; set; }
        public double[] CorrectionFactor { get; set; }
        public double Frequency { get; set; }

        public Key(string name, double[] a, double[] sigma, double[] factor)
        {
            Name = name;
            Sigma = new double[2] { sigma[0], sigma[1] };
            A = new double[2] { a[0], a[1] };
            CorrectionFactor = new double[2] { factor[0], factor[1] };
            Frequency = 0;
        }

        private double DistributionDensity(double x, double a, double sigma)
        {
            double exponentPower = (-1) * (x - a) * (x - a) / (2 * sigma * sigma);
            double exponent = Math.Exp(exponentPower);
            double divider = sigma * Math.Sqrt(2 * Math.PI);
            double fx = Math.Round(exponent / divider, 12);
            return fx;
        }

        private double ExponentDensity(double x, double m)
        {
            double a = 1 / m;
            double fx = a * Math.Exp(-a * x);
            return fx;
        }

        public double FirstPolarityProbability()
        {
            double f1, f2, p1;
            f1 = CorrectionFactor[0] == 0 ? ExponentDensity(Frequency, A[0]) : DistributionDensity(Frequency, A[0], Sigma[0]) * CorrectionFactor[0];
            f2 = CorrectionFactor[1] == 0 ? ExponentDensity(Frequency, A[1]) : DistributionDensity(Frequency, A[1], Sigma[1]) * CorrectionFactor[1];
            p1 = f1 / (f1 + f2);
            return p1;
        }

        public double SecondPolarityProbability()
        {
            double f1, f2, p2;
            f1 = CorrectionFactor[0] == 0 ? ExponentDensity(Frequency, A[0]) : DistributionDensity(Frequency, A[0], Sigma[0]) * CorrectionFactor[0];
            f2 = CorrectionFactor[1] == 0 ? ExponentDensity(Frequency, A[1]) : DistributionDensity(Frequency, A[1], Sigma[1]) * CorrectionFactor[1];
            p2 = f2 / (f1 + f2);
            return p2;
        }
    }
}
