﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Linq.Expressions;
using System.Windows.Forms;

namespace Sociometry
{
    public partial class MainForm : Form
    {
        Sociometer sociometer;

        double coincidence, mistakes;

        public MainForm()
        {
            InitializeComponent();
        }

        private void buttonMiddleValues_Click(object sender, EventArgs e)
        {
            SetAveragePolarity();
        }

        private void buttonCalcTIM_Click(object sender, EventArgs e)
        {
            if (KeyFileExist())
            {
                string interviewFilename = GetInterviewFilename();
                if (interviewFilename != null)
                {
                    MakeSocionicCalculation(interviewFilename);
                    DrawGraph();

                    string analizedFilename = Path.GetFileNameWithoutExtension(interviewFilename);
                    labelFilename.Text = analizedFilename;
                    MakeLog(analizedFilename);
                }
                else
                    MessageBox.Show("Не выбран файл интервью!");
            }
            else
                MessageBox.Show("Не найден файл ключей!");
        }

        private bool KeyFileExist()
        {
            string keyFilePath = AppDomain.CurrentDomain.BaseDirectory + "\\keys.txt";
            return File.Exists(keyFilePath);
        }

        private string GetInterviewFilename()
        {
            OpenFileDialog fileDialog = new OpenFileDialog
            {
                InitialDirectory = AppDomain.CurrentDomain.BaseDirectory,
                Filter = "Текстовые файлы|*.txt"
            };
            if (fileDialog.ShowDialog() == DialogResult.OK)
                return fileDialog.FileName;
            else
                return null;
        }

        private void MakeSocionicCalculation(string interviewFilename)
        {
            string keyFilePath = AppDomain.CurrentDomain.BaseDirectory + "\\keys.txt";
            sociometer = new Sociometer();
            sociometer.ReadParameters(keyFilePath);
            sociometer.CalcFrequency(interviewFilename);
            sociometer.CalcBayes();
            FillPolarities();
            sociometer.CalcTIM();
        }

        private void MakeLog(string filename)
        {
            string logsDir = AppDomain.CurrentDomain.BaseDirectory + $"\\logs {DateTime.Now:yy.MM.dd}";
            if (!Directory.Exists(logsDir))
                Directory.CreateDirectory(logsDir);
            string logfile = $"{logsDir}\\log {filename} {DateTime.Now:dd.MM.yyyy HH_mm_ss}.txt";
            WriteToLog(logfile, GetParameters());
            WriteToLog(logfile, GetFrequency());
            WriteToLog(logfile, GetProbabilities());
            WriteToLog(logfile, GetBayes());
            WriteToLog(logfile, GetFeaturesProbabilities());
            WriteToLog(logfile, SortFeaturesProbabilities());

            RenameLogFile(logfile);
        }

        private bool CheckValues()
        {
            const int MIN_VALUE = 0, MAX_VALUE = 100;
            foreach (TextBox textbox in panelValues.Controls)
            {
                if (textbox.Text.Trim() != "")
                {
                    try
                    {
                        double userValue = double.Parse((textbox as TextBox).Text.Trim());
                        if (userValue < MIN_VALUE || userValue > MAX_VALUE)
                        {
                            MessageBox.Show($"Значение параметра должно быть в диапазоне от {MIN_VALUE} до {MAX_VALUE}");
                            textbox.SelectAll();
                            textbox.Focus();
                            return false;
                        }
                    }
                    catch
                    {
                        MessageBox.Show("Задано неверное значение!");
                        textbox.SelectAll();
                        textbox.Focus();
                        return false;
                    }
                }
                else
                {
                    MessageBox.Show("Не задано значение параметра!");
                    textbox.Text = "   ";
                    textbox.SelectAll();
                    textbox.Focus();
                    return false;
                }
            }
            return true;
        }

        private void DrawGraph()
        {
            chartTIM.Series[0].Points.Clear();
            for (int i = 0; i < sociometer.FeaturesProbability.Length; i++)
            {
                chartTIM.Series[0].Points.AddXY(sociometer.SocionicType[i], Math.Round(sociometer.FeaturesProbability[i] * 100, 6));
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            KeysForm keysForm = new KeysForm("Экстраверсия", "Интраверсия");
            keysForm.ShowDialog();
        }

        private void button5_Click(object sender, EventArgs e)
        {
            KeysForm keysForm = new KeysForm("Интуиция", "Сенсорика");
            keysForm.ShowDialog();
        }

        private void button4_Click(object sender, EventArgs e)
        {
            KeysForm keysForm = new KeysForm("Логика", "Этика");
            keysForm.ShowDialog();
        }

        private void button3_Click(object sender, EventArgs e)
        {
            KeysForm keysForm = new KeysForm("Иррациональность", "Рациональность");
            keysForm.ShowDialog();
        }

        private void button2_Click(object sender, EventArgs e)
        {
            KeysForm keysForm = new KeysForm("Беспечность", "Предусмотрительность");
            keysForm.ShowDialog();
        }

        private void button9_Click(object sender, EventArgs e)
        {
            KeysForm keysForm = new KeysForm("Уступчивость", "Упрямство");
            keysForm.ShowDialog();
        }

        private void button8_Click(object sender, EventArgs e)
        {
            KeysForm keysForm = new KeysForm("Статика", "Динамика");
            keysForm.ShowDialog();
        }

        private void button13_Click(object sender, EventArgs e)
        {
            KeysForm keysForm = new KeysForm("Демократия", "Аристократия");
            keysForm.ShowDialog();
        }

        private void button14_Click(object sender, EventArgs e)
        {
            KeysForm keysForm = new KeysForm("Тактика", "Стратегия");
            keysForm.ShowDialog();
        }

        private void button15_Click(object sender, EventArgs e)
        {
            KeysForm keysForm = new KeysForm("Конструктивизм", "Эмотивизм");
            keysForm.ShowDialog();
        }

        private void button7_Click(object sender, EventArgs e)
        {
            KeysForm keysForm = new KeysForm("Позитивизм", "Негативизм");
            keysForm.ShowDialog();
        }

        private void button6_Click(object sender, EventArgs e)
        {
            KeysForm keysForm = new KeysForm("Рассудительность", "Решительность");
            keysForm.ShowDialog();
        }

        private void button10_Click(object sender, EventArgs e)
        {
            KeysForm keysForm = new KeysForm("Веселость", "Серьезность");
            keysForm.ShowDialog();
        }

        private void button11_Click(object sender, EventArgs e)
        {
            KeysForm keysForm = new KeysForm("Процесс", "Результат");
            keysForm.ShowDialog();
        }

        private void button12_Click(object sender, EventArgs e)
        {
            KeysForm keysForm = new KeysForm("Квестимность", "Декларативноcть");
            keysForm.ShowDialog();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            SetAveragePolarity();
        }

        private void SetAveragePolarity()
        {
            foreach (var textbox in panelValues.Controls)
            {
                (textbox as TextBox).Text = "50";
            }
        }

        private void FillPolarities()
        {
            string textBoxName;
            for (int i = 0; i < sociometer.Dihotomies.Count; i++)
            {
                textBoxName = "textBoxValue" + Convert.ToString(i + 1);
                (panelValues.Controls[textBoxName] as TextBox).Text = Convert.ToString(Math.Round(sociometer.Dihotomies[i].Bayes, 4) * 100);
            }
        }

        private void WriteToLog(string filename, string[] loggedData)
        {
            File.AppendAllLines(filename, loggedData);
        }

        private string[] GetParameters()
        {
            List<string> list = new List<string>();
            list.Add("Чтение ключей с параметрами");
            list.Add(new string('-', 10));
            for (int i = 0; i < sociometer.Dihotomies.Count; i++)
            {
                string header = $"[{sociometer.Dihotomies[i].FirstPolarityName}|{sociometer.Dihotomies[i].SecondPolarityName}]";
                list.Add(header);
                for (int j = 0; j < sociometer.Dihotomies[i].Keys.Count; j++)
                {
                    Key key = sociometer.Dihotomies[i].Keys[j];
                    list.Add($"\t{key.Name} - {key.A[0]};{key.Sigma[0]};{key.CorrectionFactor[0]};" +
                        $"{key.A[1]};{key.Sigma[1]};{key.CorrectionFactor[1]}");

                }
            }
            list.Add(new string('*', 10));
            return list.ToArray();
        }

        private string[] GetFrequency()
        {
            List<string> list = new List<string>();
            list.Add("Расчёт частот ключей");
            list.Add(new string('-', 10));
            for (int i = 0; i < sociometer.Dihotomies.Count; i++)
            {
                string header = $"[{sociometer.Dihotomies[i].FirstPolarityName}|{sociometer.Dihotomies[i].SecondPolarityName}]";
                list.Add(header);
                for (int j = 0; j < sociometer.Dihotomies[i].Keys.Count; j++)
                {
                    Key key = sociometer.Dihotomies[i].Keys[j];
                    list.Add($"\t{key.Name} - {key.Frequency}");
                }
            }
            list.Add(new string('*', 10));
            return list.ToArray();
        }

        private string[] GetBayes()
        {
            List<string> list = new List<string>();
            list.Add("Расчёт по т.Байеса:");
            list.Add(new string('-', 10));
            for (int i = 0; i < sociometer.Dihotomies.Count; i++)
            {
                string header = $"[{sociometer.Dihotomies[i].FirstPolarityName}|{sociometer.Dihotomies[i].SecondPolarityName}]";
                list.Add(header);
                list.Add($"\tБайес - {sociometer.Dihotomies[i].Bayes}");
            }
            list.Add(new string('*', 10));
            return list.ToArray();
        }

        private string[] GetProbabilities()
        {
            List<string> list = new List<string>();
            list.Add("Расчёт вероятностей:");
            list.Add(new string('-', 10));
            for (int i = 0; i < sociometer.Dihotomies.Count; i++)
            {
                string firstProbabilities = "", secondProbabilities = "";
                for (int j = 0; j < sociometer.Dihotomies[i].Keys.Count; j++)
                {
                    double firstProbability = sociometer.Dihotomies[i].Keys[j].FirstPolarityProbability();
                    firstProbabilities += $"{Convert.ToString(Math.Round(firstProbability, 4))}*";
                }
                if (firstProbabilities.Length > 0)
                    firstProbabilities = firstProbabilities.Remove(firstProbabilities.Length - 1, 1);
                for (int j = 0; j < sociometer.Dihotomies[i].Keys.Count; j++)
                {
                    double secondProbability = sociometer.Dihotomies[i].Keys[j].SecondPolarityProbability();
                    secondProbabilities += $"{Convert.ToString(Math.Round(secondProbability, 4))}*";
                }
                if (secondProbabilities.Length > 0)
                    secondProbabilities = secondProbabilities.Remove(secondProbabilities.Length - 1, 1);

                string bayes = $"bayes = {firstProbabilities}/({firstProbabilities}+{secondProbabilities})";

                string header = $"[{sociometer.Dihotomies[i].FirstPolarityName}|{sociometer.Dihotomies[i].SecondPolarityName}]";
                list.Add(header);
                list.Add($"\t{bayes}");
            }
            list.Add(new string('*', 10));
            return list.ToArray();
        }

        private string[] GetFeaturesProbabilities()
        {
            List<string> list = new List<string>();
            list.Add("Распределение вероятностей версий:");

            for (int i = 0; i < sociometer.FeaturesProbability.Length; i++)
            {
                list.Add($"{sociometer.SocionicType[i]} - {sociometer.FeaturesProbability[i]}");
            }
            list.Add(new string('_', 10));

            list.Add($"Результат: {GetMaxSocionicType()}");
            list.Add(new string('*', 10));
            return list.ToArray();
        }

        private void RenameLogFile(string filename)
        {
            double maxElement = sociometer.FeaturesProbability.Max();
            double threshold = ReadThreshold();
            if (maxElement < threshold)
            {
                RenameLogFilename(filename, "НЕТ");
            }
            else
                RenameLogFilename(filename, GetMaxSocionicType());
        }

        private string GetMaxSocionicType()
        {
            int index = 0;
            double maxElement = 0;
            for (int i = 0; i < sociometer.FeaturesProbability.Length; i++)
            {
                if (sociometer.FeaturesProbability[i] > maxElement)
                {
                    maxElement = sociometer.FeaturesProbability[i];
                    index = i;
                }
            }
            return sociometer.SocionicType[index];
        }

        private double ReadThreshold()
        {
            double threshold = 0;
            string keyFilePath = AppDomain.CurrentDomain.BaseDirectory + "\\keys.txt";
            if (File.Exists(keyFilePath))
            {
                foreach (string line in File.ReadLines(keyFilePath))
                {
                    if (line.Contains("%"))
                    {
                        threshold = Convert.ToDouble(line.Trim('%')) / 100;
                    }
                }
            }
            else
            {
                MessageBox.Show("Файл ключей не найден!");
            }
            return threshold;
        }

        private void RenameLogFilename(string filename, string resultString)
        {
            string newName = filename.Substring(0, filename.Length - 4) + " " + resultString + ".txt";
            File.Move(filename, newName);
        }

        private void buttonAllExamined_Click(object sender, EventArgs e)
        {
            labelFilename.Text = "";
            FolderBrowserDialog folderDialog = new FolderBrowserDialog
            {
                SelectedPath = AppDomain.CurrentDomain.BaseDirectory
            };
            if (folderDialog.ShowDialog() == DialogResult.OK)
            {
                string path = folderDialog.SelectedPath;
                CalculateAllExamined(path);
            }
        }

        private void CalculateAllExamined(string folderName)
        {
            string[] files = Directory.GetFiles(folderName);
            progressBar.Visible = true;
            progressBar.Maximum = files.Length;
            coincidence = 0;
            mistakes = 0;
            foreach (string filename in files)
            {
                MakeSocionicCalculation(filename);
                string analizedFilename = Path.GetFileNameWithoutExtension(filename);
                MakeLog(analizedFilename);
                CalcCoincidences(analizedFilename);
                progressBar.Value++;
            }
            MakeSummaryLog(files.Length);
            MessageBox.Show("Расчёты выполнены");
            progressBar.Visible = false;
        }

        private void MakeSummaryLog(int count)
        {
            string logsDir = AppDomain.CurrentDomain.BaseDirectory + $"\\logs {DateTime.Now:yy.MM.dd}";
            if (!Directory.Exists(logsDir))
                Directory.CreateDirectory(logsDir);
            string summaryLog = $"{logsDir}\\summary.txt";
            double versionNo = CheckResearchResults(logsDir)[0];
            double coincidenceCount = CheckResearchResults(logsDir)[1];
            double mistakeCount = CheckResearchResults(logsDir)[2];
            File.AppendAllText(summaryLog, $"Порог срабатывания - {ReadThreshold() * 100}%\n");
            File.AppendAllText(summaryLog, $"Число версий - {count - versionNo}\n");
            File.AppendAllText(summaryLog, $"\n");
            File.AppendAllText(summaryLog, $"Версии НЕТ - {versionNo}\n");
            File.AppendAllText(summaryLog, $"Совпало - {coincidenceCount}\n");
            File.AppendAllText(summaryLog, $"Ошибки - {mistakeCount}\n");
            File.AppendAllText(summaryLog, $"Всего обработано - {count}\n");
            File.AppendAllText(summaryLog, $"{new string('-', 10)}\n");

            File.AppendAllText(summaryLog, $"Распознавание - {(coincidenceCount / count) * 100:0.00}\n");
            File.AppendAllText(summaryLog, $"Сходимость - {(coincidenceCount / (count - versionNo)) * 100:0.00}\n");
            File.AppendAllText(summaryLog, $"{new string('-', 10)}\n");

            File.AppendAllText(summaryLog, $"Попавшие в 1-ую maxrange={GetMaxRange()} - {(coincidence / count) * 100:0.00}\n");
        }

        private double[] CheckResearchResults(string logsFolderName)
        {
            string[] files = Directory.GetFiles(logsFolderName);
            double versionNo = 0, coincidenceCount = 0, mistakeCount = 0;
            string typeKey, typeValue;
            SocionicTypes socionicTypes = new SocionicTypes();
            foreach (string file in files)
            {
                string filename = Path.GetFileNameWithoutExtension(file);
                if (filename.Contains("НЕТ"))
                {
                    versionNo++;
                    continue;
                }
                else
                {
                    typeKey = filename.Substring(4, 4);
                    typeValue = filename.Substring(filename.LastIndexOf(" ") + 1);
                    if (typeValue == SocionicTypes.Types[typeKey])
                        coincidenceCount++;
                    else
                        mistakeCount++;
                }
            }
            return new double[] { versionNo, coincidenceCount, mistakeCount };
        }

        private string[] SortFeaturesProbabilities()
        {
            List<string> list = new List<string>
            {
                "Отсортированные распределения вероятностей версий:"
            };

            Dictionary<string, double> sortedTypes = sociometer.GetSortedTypes();
            foreach (KeyValuePair<string, double> keyValuePair in sortedTypes)
            {
                list.Add($"{keyValuePair.Key} - {keyValuePair.Value}");
            }

            return list.ToArray();
        }

        private void CalcCoincidences(string filename)
        {
            string socionicKey = filename.Substring(0, 4);
            string socionicValue = SocionicTypes.Types[socionicKey];
            Dictionary<string, double> sortedTypes = sociometer.GetSortedTypes();
            int i = 0;
            int maxRange = GetMaxRange();
            string[] bestTypes = new string[maxRange];
            foreach (var item in sortedTypes)
            {
                if (i == maxRange)
                    break;
                bestTypes[i] = item.Key;
                i++;
            }
            bool coincidenceFlag = false;
            for (int j = 0; j < maxRange; j++)
            {
                if (socionicValue == bestTypes[j])
                {
                    coincidence++;
                    coincidenceFlag = true;
                    break;
                }
            }
            if (coincidenceFlag == false)
                mistakes++;
        }

        private int GetMaxRange()
        {
            int maxRange = 1;
            string keyFilePath = AppDomain.CurrentDomain.BaseDirectory + "\\keys.txt";
            if (File.Exists(keyFilePath))
            {
                foreach (string line in File.ReadLines(keyFilePath))
                {
                    if (line.Contains("maxrange"))
                    {
                        try
                        {
                            maxRange = Convert.ToInt32(line.Substring(line.IndexOf('=') + 1));
                        }
                        catch
                        {
                            maxRange = 1;
                        }
                    }
                }
            }
            else
            {
                MessageBox.Show("Файл ключей не найден!");
            }
            return maxRange;
        }
    }
}
